unit GameServer;

interface

Uses GameClient;    

Type
    TGameServerList=Class;
    TGameServerVariable=Class;
    TGameServerListVariable=class(TVariable)
    end;
    TGameClientArray = Array of TGameClient;
    TGameServer=Class(TObjectNode)
    public
        Property    GameClient[I:integer]: TGameClient Read GetGameClient;
        Property    GameType: Integer Read GetGameType Write SetGameType;
        Property    Variable: TGameServerVariable Read GetVar;

        // List acquired with GetPlayerList MUST be released with ReleaseList
        Function    GetPlayerList: TGameClientArray;
        Function    FindByName(Name: String): TGameClient;
        Function    FindByID(ID: Integer): TGameClient;
        Function    FindByUserID(ID: Integer): TGameClient;
        Function    FindByUniqueID(ID: String): TGameClient;

        // Run a trigger
        Procedure   RunTrigger(Trigger: String);

        // Get variable value in order 1) self, 2) GameServerList
        Function    GetString(Name: String; DefaultValue: String=''):string;
        Function    GetInteger(Name: String; DefaultValue: Integer=0): Integer;

        // Do Status
        Function    DoStatusBlocking:String;
        Function    DoStatusNonBlocking:String;

        // Do Rcon Command
        Function    DoRcon(Cmd: String; Var Response:String):Boolean;

        // Do non blocking rcon command
        Function    DoRconNonBlocking(Cmd: String; Var Response:String):Boolean;
        Function    DoRconNonBlockingList(Const Cmd: TStringArray; Var Response:String):Boolean;

        // Do Rcon Command
        Function    DoSay(Msg: String; Var Error: String):Boolean;

        // Return settings
        Function    Settings:String;

        Function    Run(Const Cmd: TStringArray; Const Params: TStringArray; ParamStr: String):string;
        Function    Help: String;

        Constructor Create;
        Destructor  Destroy;
    end;

    TGameServerList=Class(TObjectNode)
        Property    Variable: TGameServerListVariable Read GetVar;
        Property    GameServer[I:integer]: TGameServer Read GetGameServer;
        Function    FindByName(Name: String): TGameServer;
        Function    FindByID(ID: Integer): TGameServer;
        Function    FindByAddress(Const Address: String): TGameServer;
        Function    GetPath:String;
        // Get settings
        Function    Settings:String;
        // Get Help
        Function    Help: String;
        Function    Run(Const Cmd: TStringArray; Const Params: TStringArray; ParamStr: String):string;
    end;

    TGameServerVariable=Class(TVariable)
    end;

Const
    GS_SERVERNAME = 'ServerName';
    GS_VERSION = 'Version';
    GS_PUBLICADDRESS = 'PublicAddress';
    GS_MAP = 'Map';
    GS_PLAYERSSTRING = 'PlayersString';

    GS_AUTHED = 'Authed';
    GS_UNAUTHED = 'Unauthed';
    GS_ONAUTHED = 'OnPlayerAuthed';
    GS_ONUNAUTHED = 'OnPlayerUnauthed';
    GS_ONSTATUS = 'OnStatus';
    GS_ONWRONGPASSWORD = 'OnWrongPassword';
    GS_ONBANNED = 'OnBanned';

    GS_GAMETYPE = 'GameType';
    GS_PORT = 'Port';
    GS_IP = 'IP';
    GS_RCONPASSWORD = 'RconPassword';
    GS_PASSWORD = 'Password';

    GS_SAYCOMMAND = 'SayCommand';
    GS_SAYWIDTH = 'SayWidth';

    RCON_FAILED=#0#0#0#0'Failed';
    RCON_UNSUPPORTED_GAMETYPE='Unsupported game type';

    DEFAULT_SAYWIDTH = 63;
    DEFAULT_SAYCOMMAND = 'Say';

Function GetGameServerList: TGameServerList;

implementation
end.
