unit ObjectNode;

interface

Uses Variables;

Const NO_EXECUTE_REPLY:String = #0#0#0#0;

Type
    TObjectNode = Class;
    TObjectLeaf = Class;

    TObjectLeaf = Class
    public
        Property    Name: String Read FName Write SetName;
        Property    Parent: TObjectNode Read FParent;
        Property    Root: String Read FRoot;
        Property    Path: String Read GetPath;
        Property    Enabled: Boolean Read FEnabled Write SetEnabled;
        Property    Id: Integer Read FId;

        // Write to log, if it exists
        Procedure   Writeln(Line: String; MessageType: Integer=0); Virtual;

        // Info about this object
        Function    Info: String; Virtual;

        // All help
        Function    Help: String; Virtual;

        // Get help about a command
        Function    GetHelp(Cmd: String): String; Virtual;

        Function    Settings: String; Virtual;

        // Execute a command
        Function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; Virtual;

        // Log = Create Logger object, Variable = Create Variable object
        Constructor Create(CreateLog:Boolean=false; CreateVariable:TVariableClass=nil);
        Destructor  Destroy; 
    end;

    TObjectArray = Array Of TObjectLeaf;
    TObjectNode = Class(TObjectLeaf)
    public
        Function    FindObjectByID(Id:Integer): TObjectLeaf;
        Function    FindObjectByName(Name: String): TObjectLeaf;
        Procedure   Remove(Obj: TObjectLeaf);
        Function    Add(Obj: TObjectLeaf):Boolean;
        Function    Has(Obj: TObjectLeaf):Boolean;
        Procedure   Clear;
        Function    GetHelp(Cmd: String): String; 
        Function    Help: String; 
        Function    Count: Integer;
        Function    Settings: String; 

        Function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; 
        Constructor Create(CreateLog:Boolean=false; CreateVariable:TVariableClass=nil);
        Destructor  Destroy; 
    end;

// If Run command returns this value, the object was detroyed on the call
Const RS_DESTROYED = 'Destroyed';

implementation
end.
