unit Reader;
(* Reader of inputs *)
interface

Uses
    Enviroment;

Type
    TReaderList = Class;
    TReader = Class;

    TReaderList = Class
        Public
            Property    Reader[I:integer]: TReader Read GetReader;
            Property    Path: String Read FPath Write SetPath;
            Function    Count: Integer;
            function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string;

            Function    FindByID(Id:Integer): TReader;
            Function    FindByName(Name: String): TReader;
            // Will be cleared when all threads have finished
            Procedure   Clear;

            // Get Id for new Reader
            Function    GetNewId: Integer;
            Function    Settings: String;

            Function    Add(R:TReader):boolean;
            Function    Has(R:TReader):Boolean;
            Procedure   Remove(R:TReader);

            Constructor Create;
            Destructor  Destroy; 
    end;

    TReader = Class
        Public
            // This reader is in the middle of execution right now
            Property    Enabled: Boolean Read FEnabled Write SetEnabled;
            Property    Enviroment: TEnviroment Read FEnviroment;
            Property    LinesSent: Integer Read FLinesSent;
            Property    LinesRecv: Integer Read FLinesRecv;
            Function    Path: String;
            Property    Id: Integer Read FId;
            Property    Name: String Read FName Write SetName;
            Property    Parent: TReaderList Read FParent;
            Function    Info: String; Virtual;

            Function    Settings: String; Virtual;
            Function    Statistics: String;

            // Blocking read call
            Function    Readln: String; Virtual;

            // For replying
            Procedure   Writeln(Line: String);
            function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; Virtual;
            Constructor Create;
            Destructor  Destroy; 
    end;

    // A Reader with connection
    TReaderConnection = Class( TReader )
        Public
            Constructor Create;
            Destructor  Destroy; 
    end;

    TReaderStdIn = Class( TReaderConnection )
        Public
            // For replying
            Constructor Create;
            Destructor  Destroy;
            Function    Info: String;
            Function    Settings: String; 
    end;

    TReaderTcp = Class( TReaderConnection )
        Public
            Procedure   DisconnectAll;
            Function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; 

            // For replying
            Function    Info: String;
            Function    Settings: String; 

            Constructor Create(Port: Integer; BindHost: String='');
            Destructor  Destroy; 
    end;

    TReaderTcpConnection = Class( TReaderConnection )
        Public
            Property    PeerIp: String Read FPeerIp;
            Property    Ip: String Read FIp;
            Property    Port: Integer Read FPort;
            Property    PeerPort: Integer Read FPeerPort;

            // For replying
            Function    Info: String;
            Function    Settings: String;
            Destructor  Destroy; 
    end;

    // For replying
    TReaderUdp = Class( TReader )
        Public
            Function    Info: String;
            Function    Settings: String; 

            Constructor Create(Port: Integer; BindHost: String='');
            Destructor  Destroy; 
    end;

    TReaderFile = Class( TReaderConnection )
        public
            Function    Settings: String; 
            Function    Info: String; 

            Constructor Create(Filename:String);
            Destructor  Destroy; 
    end;

    TReaderDaemon = Class( TReaderConnection )
        public
            Function    Settings: String; 
            Function    Info: String; 
            Constructor Create(Command:String);
    end;

Function GetReaderList: TReaderList;

implementation
end.
