unit Writer;
(* Writer of logs *)
interface

Const
    (* Message type, These are used currently only in sql-writing *)
    MT_DEFAULT=0;
    MT_READER=1;    // Reader Reply packet
    MT_READER_IN=2;
    MT_READER_OUT=3;
    MT_UNHANDLED_EXCEPTION=4;
    MT_SSC=1001;
    MT_SSC_EXCEPTION=1002;
    MT_SSC_ERROR=1002;
    MT_SSC_SCAN=1003;
    MT_SSC_STATUS=1003;
    MT_SSC_CHECKSUM=1004;
    MT_SSC_SHOT=1005;
    MT_SSC_DISCONNECT=1006;
    MT_SSC_CONNECT=1007;
    MT_SSC_SYSTEMID=1008;
    MT_FTP_MSG=1100;
    MT_SCRIPT_ERROR=1201;
    MT_SCRIPT=1200;

Type
    TWriter=Class;
    TWriterList=Class
        public
            Property    Writer[I:integer]: TWriter Read GetWriter;
            Property    Path: String Read FPath Write FPath;
            Procedure   Writeln(Line: String; MessageType: Integer=MT_DEFAULT);

            Function    FindByID(Id:Integer): TWriter;
            Function    FindByName(Name: String): TWriter;
            Function    Add(R:TWriter):boolean;
            Function    Settings: String;

            Function    Count: Integer;
            Function    Info:String;

            function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; Virtual;
            Constructor Create;
            Destructor  Destroy; 
    end;

    TWriter=Class
        public

        Property    Path: String Read GetPath;
        Property    TimeStampString: String Read FTimeStampString Write FTimeStampString;
        Property    TimeStamp: Boolean Read GetTimeStamp Write SetTimeStamp;
        Property    Id: Integer Read FId Write FId;
        Property    Name: String Read FName Write FName;
        Property    Enabled: Boolean Read FEnabled Write FEnabled;

        function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string; Virtual;

        Function    Settings: String; Virtual;
        Procedure   Writeln(Line: String; MessageType: Integer=MT_DEFAULT);
        Function    Info:String; Virtual;
        Constructor Create;
    end;

    TWriterStdOut=Class(TWriter)
    end;

    TWriterUdp=Class(TWriter)
    end;

    TWriterMySql=Class(TWriter)
        public
        Constructor Create(
            ServerList:String;
            QueryString:String='INSERT INTO %table (time, messagetype, message) VALUES(NOW(), %messagetype, %message)';
            CreateTable:String='CREATE TABLE %table (id int(11) not null auto_increment, time datetime, messagetype int(11), message varchar(255), primary key (id))');
        // ServerList = host,port,user,pass,db[,table]:...
        // QueryString = %table "%message" %messagetype "%time"
    end;

    TWriterFile=Class(TWriter)
        public
        Constructor create(filename: String);
    end;

    TWriterRcon=Class(TWriter)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

    TWriterHLRcon=Class(TWriterRcon)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

    TWriterETRcon=Class(TWriterRcon)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

    TWriterMOHAARcon=Class(TWriterRcon)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

    TWriterCODRcon=Class(TWriterRcon)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

    TWriterQ3Rcon=Class(TWriterRcon)
        public
        Constructor create(Server: String; Port:Integer; Password:String);
    end;

Function GetWriterList: TWriterList;

implementation
end.
