unit ssClient;

interface

Uses
    StringFunctions,
    Enviroment,
    Writer, Variables,
    ObjectNode,
    GameTypes,
    GameClient,
    ScreenShot,
    StringPair;

Type
    TssClientVariables=Class;

    TShotStatus = (sNone, sDownloading, sComplete);
    TssClient=Class(TObjectLeaf)
    Public
        // Search variable value of self
        // This is used to get values of this client, like IP, name ect
        Property    Variable: TssClientVariables Read GetVar;
        // Search variable value in order 1) self, 2) ssServerList
        // this is used for finding event values (or recursively anything)
        Function    GetValue(Name: String):String;

        // Player that has been identified as same user as this client
        Property    GameClient: TGameClient Read FGameClient Write SetGameClient;
        // Heuristic value of this Client/Player binding
        Property    GameClientValue: Integer Read FGameClientValue Write FGameClientValue;
        // The server this client is on, or was last known
        Function    GetGameServer: TObject;
        // Save shot, Variables: %index %time %date %timedate %name %ip %gameservername %serverport %port (ssserver)
        // Returns the filename
        Function    SaveShot(Const Filename: String):String;
        // Write to logger
        Procedure   Writeln(Line: String; MessageType: Integer=MT_SSC); 
        // Info about this object
        Function    Info: String; 
        // Get ssServer
        Function    ssServer: TObject;
        // Run a trigger
        Procedure   RunTrigger(Const Trigger: String);
        // Gets configure variable. tries in this order 1) ssServer 2) gameserver 3) gameserverlist
        Function    ConfigString(Name: String; DefaultValue: String=''): String;
        Function    ConfigInteger(Name: String; DefaultValue: Integer=0): Integer;
        Function    Settings: String; 
        //------------------------------------------------------------------//
        // All help
        Function    Help: String; 
        // Execute a command
        Function    Run(Const Cmd, Params: TStringArray; ParamStr: String): string;
        //------------------------------------------------------------------//
        // Is this client authed
        Function    IsAuthed: Boolean;
        // Set Title
        Procedure   SetTitle(Title: String);
        // Write to client
        Procedure   Write(Line: String);
        // Send banner to client
        Procedure   SendBanner(Const filename: String);
        // Send banner to client
        Procedure   SendBannerURL(Const URL: String);
        // Send info string to client
        Procedure   SendInfo(Const Line: Integer; Const Msg, Url, Tip: String);
        //------------------------------------------------------------------//
        // Queries info about client, All but sound/process scans
        Procedure   QueryStatus(Flags: Integer=$FA7F);
        // Queries info about client, All but sound/process scans
        Procedure   QueryA3D;
        // Full scan returns all processes running and all libraries attached to game
        Procedure   QueryProcesses(FullScan: Boolean = False);
        // Does Custom file scan
        // (Works currently only with half-life)
        // There is a problem on finding the base directory of other games
        // On half-life all steam/steamapps/ are searched
        // Subdirector for CS/Sound would be '\counter-strike\cstrike\sound\'
        Procedure   QueryCustomSounds(SubDirectory: String);
        // Ask process checksum from the client
        Procedure   QueryProcessChecksum;
        // Get list of hooks
        Procedure   ScanCheats(Const SearchFor: TStringArray; FastScan:Boolean; DelayedScan:Boolean);
        //------------------------------------------------------------------//
        // Ask client to disconnect
        Procedure   RequestDisconnect;
        // Ask client to download another version
        Procedure   RequestToDownloadVersion(Version:Integer);
        // Ask client to take a screenshot
        Procedure   RequestScreenshot;
        // Asks to take a screenshot if clients shots per hour value is less than
        // Given parameter
        Function    RegulatedScreenshots(ShotsPerHour: Double):boolean;
        //------------------------------------------------------------------//
        // Send ping to client
        Procedure   SendPing;
    end;

    TssClientVariables=Class(TVariable)
    public
        Function    IsVariableAllowed(Name, Value:String):String; 
    end;

Const
    (* Client variables *)
    SC_NAME = 'Name';              // Name on game server
    SC_IDCLIENT = 'IdClient';      // SteamID, Client tells
    SC_IDSERVER = 'IdServer';      // SteamID, Server tells
    SC_IP = 'IP';                  // Client IP
    SC_PORT = 'Port';              // Client clientPort
    SC_SERVERIP = 'ServerIP';      // ssServer IP
    SC_SERVERPORT = 'ServerPort';  // ssServer clientPort
    SC_WINE = 'Wine-mode';         // Wine emulation mode?
    SC_GAMEACTIVE = 'GameActive';  // Is Client's game active (foreground)
    SC_GAMERUNNING = 'GameRunning';// Is Client's game running (executed)
    SC_GAMETYPE = 'GameType';      // Game type

    SC_SYSTEMID = 'SystemId';      // SSClient ID
    SC_ONSYSTEMID = 'OnSystemId';  // When ssClient ID is known

    SC_ONAUTHED = 'OnAuthed';      // Unauthed client
    SC_ONUNAUTHED = 'OnUnauthed';  // Authed client

    SC_UNIQUEID = 'UniqueId';     // Last known Unique ID (SteamID)
    SC_ONCONNECT = 'OnConnect';   // When player connects to ssServer
    SC_ONDISCONNECT = 'OnDisconnect'; // When player quits, connection is already lost
    SC_ONVERSION = 'OnVersion';     // Client tells full version string

    SC_PINGINTERVAL = 'PingInterval';    // How often send Pings
    SC_DEFAULT_PINGINTERVAL = 10;
    SC_MAXPING = 'MaxPing';     // Maximum Ping allowed
    SC_DEFAULT_MAXPING = 10;
    SC_ONMAXPING = 'OnMaxPing'; // Trigger to fire when ping exceeds max range
    SC_LASTPING = 'Ping';   // Last Ping value
    SC_ONCONNTRACKERUNAVAILABLE = 'OnConnTrackerUnavailable';

    SC_SHOT_CAPTURED = 'ShotsCaptured';        // Number of shots captured
    SC_SHOT_DOWNLOADED = 'ShotsDownloaded';    // Number of shots Downloaded
    SC_SHOT_CAPTUREMETHOD = 'ShotCaptureMethod';    // Capture method 0/1
    SC_SHOT_RATE = 'ShotLastTransferRate';    // The rate shot was uploaded
    SC_SHOT_ONCAPTURED = 'OnShotCaptured';    // When client has captured a shot %rate
    SC_SHOT_ONDOWNLOADED = 'OnShotDownloaded';// When shot has been downloaded   %index %time %date %timedate %name %ip %gameservername %serverport %port %uniqueid (ssserver)
    SC_SHOT_ONTOOSLOW = 'OnMinimumRate';      // When Download is too slow       %rate
    SC_SHOT_LASTSHOTFILE = 'LastShotFile';    // Name of last file that was written on HD

    SC_PROCESSCHECKSUM = 'ProcessChecksum';         // ssClient's process checksum
    SC_FILECHECKSUM = 'FileChecksum';               // ssClient file checksum
    SC_LEGALPROCESSCHECKSUM = 'LegalProcessChecksum';    // Allowed Process checksum
    SC_LEGALFILECHECKSUM = 'LegalFileChecksum';          // Allowed file checksum
    SC_ONILLEGALPROCESSCHECKSUM = 'OnIllegalProcessChecksum';    // On Wrong process checksum
    SC_ONILLEGALFILECHECKSUM = 'OnIllegalFileChecksum';          // On Wrong File checksum

    SC_CHEATNAME = 'CheatName';    // Cheat Name
    SC_CHEATTYPE = 'CheatType';    // Cheat Type

    SC_TIME_CONNECTED = 'TimeConnected';   // When client connected
    SC_VERSIONSTR = 'VersionString';   // Version string given later
    SC_VERSION = 'Version';        // Version that was given in negoation

    SC_CUSTOMDLL = 'CustomDLLs';   // Files loaded into game

    SC_PRECACHEPOSSIBLE = 'PrecachePossible';       // Game Was Started before ssc
    SC_CUSTOMFILES = 'CustomFiles';         // Files in game dir
    SC_ONCUSTOMFILES = 'OnCustomFiles';     // Trigger called when custom files list is complete

    SC_ONILLEGALFILE = 'OnIllegalFile';     // Trigger with for each illegal file var.FoundIllegalFile var.FoundIllegalFileSize var.FoundIllegalWavLength var.FoundIllegalFileChecksum
    SC_ILLEGALFILE = 'FoundIllegalFile';    // Illegal files that was found
    SC_ILLEGALFILESIZE = 'FoundIllegalFileSize'; // Size of illegal file
    SC_ILLEGALWAVLENGTH = 'FoundIllegalWavLength'; // Length of wav, in ms
    SC_ILLEGALFILECHECKSUM = 'FoundIllegalFileChecksum'; // Checksum of illegal file
    SC_ILLEGALFILETIME = 'FoundIllegalFileTime'; // Time of illegal file

    SC_ONILLEGALFILES = 'OnIllegalFiles';   // Trigger with list of all illegal files var.FoundIllegalFiles
    SC_ILLEGALFILES = 'FoundIllegalFiles'; // Illegal files that were found

    SC_A3D = 'a3d';                 // A3D
    SC_ONA3D = 'OnA3D';             // Triggered when client has ran A3D
    SC_W9X = 'w9x';                 // Client running windows 9x


implementation
end.
