(*
This script requires you have one of the following server side modules
installed on your half-life server (hlguard, statsme, amx, amxx or mani admin mod)
You must also set the gameserver.<server>.servermod variable
Example: gameserver.<server>.servermod hlg

Or you can also set a global default for all servers
Example: gameserver.var.servermod hlg

Supported values are: hlg, sm, amx, amxx, mani

To load this script execute the following commands

Script.unload nameprefix
Script.run nameprefix scripts/nameprefix.pas
Script.nameprefix.log.add console stdout
Script.nameprefix.log.console.enable
 *)
program nameprefix;

Const PREFIX = '[NO-SSC]';
    SERVER_MOD_VARIABLE = 'servermod';
    MOD_HLG = 'hlg';
    MOD_SM = 'sm';
    MOD_AMX = 'amx';
    MOD_AMXX = 'amxx';
    MOD_MANI = 'mani';

Function IsNamePrefixed(Name: String): Boolean;
begin
    Result := InsensitiveCompare(Copy(Name, 1, Length(PREFIX)), PREFIX);
end;

Procedure RenamePlayer(Server: TGameServer; Player: TGameClient; NewName: String);
Var Cmd, Response: String;
    ServerMod: String;
    UserID: String;
begin
    ServerMod := lowercase(Server.GetString(SERVER_MOD_VARIABLE, ''));

    UserID := Player.Variable.VarString[GC_USERID];
    If ServerMod = MOD_HLG then
        Cmd := 'hlg_clexec #'+UserID+' "name \'''+NewName+'\''"'
    else If ServerMod = MOD_AMX then
        Cmd := 'amx_name #'+UserID+' "'+NewName+'"'
    else If ServerMod = MOD_AMXX then
        Cmd := 'amx_nick #'+UserID+' "'+NewName+'"'
    else If ServerMod = MOD_SM then
        Cmd := 'sm_do "sm_clexec2 #'+UserID+' \''name \\'''+NewName+'\\''\''"'
        // Is there security risk here? Can player send commands to statsme by changing his name??
    else If ServerMod = MOD_MANI then
        Cmd := 'ma_cexec '+UserID+' name '+NewName+''
    else exit;

    // Write to console for debugging purposes
    Writeln(Cmd);

    // Send rcon message
    Server.DoRconNonBlocking(Cmd, Response);
end;

Procedure   OnPlayerAuthed(Client: TssClient; Player: TGameClient; Server: TGameServer);
var Name: String;
begin
    Name := Player.Variable.VarString[GC_NAME];
    If not IsNamePrefixed(Name) then exit;
    // Player is authenticated but still prefixed "[NO-SSC]player"
    // Remove the prefix "[NO-SSC]player" -->> "player"
    Name := Copy(Player.RealName, Length(PREFIX)+1, Length(Name)-Length(PREFIX)+1);
    RenamePlayer(Server, Player, Name);
end;

Procedure   OnPlayerUnauthed(Client: TssClient; Player: TGameClient; Server: TGameServer);
var Name: String;
begin
    If not Player.Variable.VarBoolean[GC_ONLINE] then exit;
    Name := Player.RealName;
    If IsNamePrefixed(Name) then exit;
    // Player is not authed and it's name is not prefixed.
    // Change name "Player" -->> "[NO-SSC]Player"
    Name := PREFIX+Name;
    RenamePlayer(Server, Player, Name);
end;

Procedure ShowServerInfo;
Var Servers: TGameServerList;
    Server: TGameServer;
    ServerMod: String;
    Line: String;
    AllOk: Boolean;
    ExampleServer: String;
    I: Integer;
begin
    AllOk := true;
    Servers := GetGameServerList;
    For I:=0 to Servers.Count-1 do begin
        Server := Servers.GameServer[I];
        ServerMod := lowercase(Server.GetString(SERVER_MOD_VARIABLE, ''));
        Line := '['+Inttostr(Server.id)+'] '+Server.name+' ';
        If ServerMod = MOD_HLG then Line := Line + 'HLGuard' else
        If ServerMod = MOD_AMX then Line := Line + 'AMX' else
        If ServerMod = MOD_AMXX then Line := Line + 'AMXX' else
        If ServerMod = MOD_SM then Line := Line + 'StatsMe' else
        If ServerMod = MOD_MANI then Line := Line + 'Mani Admin Plugin' else
        begin
            Line := Line +' SERVER SIDE MODULE NOT SELECTED, NamePrefix disabled';
            AllOK := False;
            ExampleServer := Server.name;
        end;
        Writeln(Line);
    end;

    If not ALLOK then begin
        Writeln('For nameprefix script to work you must specify serverside module for all game servers');
        Writeln('Supported modules are '+MOD_HLG+', '+MOD_SM+', '+MOD_AMX+', '+MOD_AMXX+' and '+MOD_MANI);
        Writeln('Example: gameserver.'+ExampleServer+'.'+SERVER_MOD_VARIABLE+' hlg');
        Writeln('Or you can also specify global default variable for all gameservers if all your the game');
        Writeln('servers run the same plugins');
        Writeln('Example: gameserver.var.'+SERVER_MOD_VARIABLE+' hlg');
    end;
end;

begin
    Writeln('Nameprefix script loaded. Prefix is "'+PREFIX+'"');
    ShowServerInfo;

end.
