(*
 This example creates Half-life remote connection server.
 With this script player can send rcon commands to ssServer.

 This script starts listening rcon commands on port 27030.
 In half-life use commads:
    rcon_address <ssServer address>
    rcon_port 27030
    rcon help

 This script authenticates player by IP:Port pair. Not too secure, but because
 rcon has challenge system it is adequate.

 To load this script into ssServer run:
script.load rconserver scripts/rconserver.pas
script.rconserver.log.add console stdout
script.rconserver.log.add file log/rconserver.log
script.rconserver.execute 27030
 *)
program rconserver;

var RconServer: TRconServer;

Procedure Finalization;
begin
    Writeln('Unloading the script');
    DestroyRconServer(RconServer);
end;

Procedure Initialization;
var Port: Integer;
begin
    If ParamCount=0 then begin
        Writeln('You must specify port number as parameter');
        exit;
    end;
    Port := StrToInt(ParamStr(1));
    RconServer := CreateHLRconServer(Port,'')
    if (RconServer=nil) then
    begin
        Writeln('Failed to open port '+IntToStr(port)+' for listening');
    end else
    begin
        Writeln('Listening RCon commands on port '+IntToStr(port));
    end;
end;

// Find the player with IP and Port
Function FindPlayer(IP:String; Port: Integer): TGameClient;
var GSList: TGameServerList;
    GS: TGameServer;
    GC: TGameClient;
    I, J:Integer;
begin
    Result := nil;
    
    GSList := GetGameServerList;
    For I:=0 to GSList.count-1 do
    begin
        GS := GSList.GameServer[I];
        For J:=0 to GS.count-1 do
        begin
            GC := GS.GameClient[J];
            If (GC.Variable.VarString[GC_IP]=IP) and
               (GC.Variable.VarInteger[GC_PORT]=Port) then
            begin
                Result := GC;
                exit;
            end;
        end;
    end;
end;

Procedure OnRConCommand(Sender: TRconServer; Password, Command, IP: String; Port: Integer; var Response: String);
var GC: TGameClient;
    SteamID: String;
begin
    command := trim(command);
    Writeln(IP+':'+inttostr(port)+': sent rcon "'+Command+'"');
    GC := FindPlayer(IP, Port);
    If (GC=nil) then begin
        Response := 'Who the hell are you?';
        exit;
    end;

    SteamID := GC.UniqueID;
    Response := 'Hello '+GC.RealName+' your Steam_ID is '+SteamID;
end;

begin
end.
