(*
 This example creates Half-life remote connection server.
 With this script player can send rcon commands to ssServer.

 This script starts listening rcon commands on port 27030.
 In half-life use commands:
    rcon_address <ssServer address>
    rcon_port 27030
    rcon_password <password>
    rcon help

 OR to use rcon securely over ssclient tunneling, run in half-life commands:
    rcon_address 127.0.0.1
    rcon_port 27030
    rcon_password <password>
    rcon help

 If half-life 2 / Counter-strike: Source use the following command:
    rcon_address localhost:27030
    rcon_password <password>
    rcon help

 So to use this script
  1) start ssc
  2) go to a configured game server
  3) set rcon_address and rcon_port to ssServer
  4) rcon help

 To configure authenticated admins run:
 gameserver.<name>.var.set SandboxAdmins "STEAM_0:0:224438 STEAM_0:0:XXXX STEAM_0:0:YYY STEAM_0:0:ZZZ"

 Or to configure admins on all servers run:
 gameserver.all.var.set SandboxAdmins "STEAM_0:0:224438 STEAM_0:0:XXXX STEAM_0:0:YYY STEAM_0:0:ZZZ"

 You must also set rcon password 
 gameserver.<name>.var.set SandboxPassword "Password"

 To load this script into ssServer run:
gameserver.all.var.set SandboxAdmins "STEAM_0:0:XXX STEAM_0:0:224438"
gameserver.all.var.set SandboxPassword "password"
script.unload sandbox
script.load sandbox scripts/sandbox.pas
script.sandbox.log.add console stdout
script.sandbox.log.add filelog file log/sandbox.log
script.sandbox.execute 27030
 *)
program sandbox;

Const VAR_ADMINLIST = 'SandboxAdmins';
Const VAR_ADMINPASSWORD = 'SandboxPassword';

var RconServer: TRconServer;   // Half-life 1 rcon server
var Rcon2Server: TRconServer;  // Half-life 2 rcon server

Procedure Finalization;
begin
    Writeln('Unloading the script');
    DestroyRconServer(RconServer);
    DestroyRconServer(Rcon2Server);
end;

Procedure Initialization;
var Port: Integer;
begin
    If ParamCount=0 then begin
        Writeln('You must specify port number as parameter');
        exit;
    end;
    Port := StrToInt(ParamStr(1));

    RconServer := CreateHLRconServer(Port,'');
    if (RconServer=nil) then
    begin
        Writeln('Failed to open port '+IntToStr(port)+'/UDP for listening');
    end else
    begin
        Writeln('Listening RCON commands on port '+IntToStr(port)+'/UDP');
    end;

    Rcon2Server := CreateHL2RconServer(Port,'');
    if (Rcon2Server=nil) then
    begin
        Writeln('Failed to open port '+IntToStr(port)+'/TCP for listening');
    end else
    begin
        Writeln('Listening RCON commands on port '+IntToStr(port)+'/TCP');
    end;


end;

Procedure KickPlayer(Player: TGameClient; Kicker: TGameClient; Var Response: String);
begin
    If Player=nil then begin
        Response := Response + 'Player not found, cannot kick the player'#13#10;
        exit;
    end;
    Response := Response + Execute(Player.path+'.kick "Kicked by '+Kicker.Realname+'"'#13#10, nil);
end;

Procedure TakeShot(Player: TGameClient; Var Response: String);
var SC: TSSClient;
begin
    If Player=nil then begin
        Response := Response + 'Player not found, cannot take a screen shot'#13#10;
        exit;
    end;
    SC := TSSClient(Player.ssClient);
    If SC=nil then begin
        Response := Response + Player.name+' is not running ssClient, cannot take a screen shot'#13#10;
        exit;
    end;
    Response := Response + SC.name+' ('+Player.UniqueID+') '+execute('ss.['+IntToStr(SC.Id)+'].shot',nil)+#13#10;
end;

Function Help: String;
begin
    Result :=
'ssServer menu:'+#13#10+
'kick enable      Enable kicking of unauthed players'+#13#10+
'kick disable     Disable kicking of unauthend players'+#13#10+
'kick status      Tells wheather the kicking is enabled'+#13#10+
'kick <name|id>   Kick a player from the server. Example: kick *clan*'+#13#10+
'Restart          Sends sv_restart 1'+#13#10+
'Authed           Information about authed players'+#13#10+
'shot <name|id>   Take a screenshot on a player. Example: shot *clan*??*'+#13#10+
'map <map_name>   Changes the level'+#13#10+
'say <message>    Say message on the server'+#13#10+
'rcon <command>   Send rcon command'+#13#10+
'';
end;

Procedure HandleClient(GC: TGameClient; GS: TGameServer; Command, Password: String; Var Response: String);
var UniqueID: String;
    Tmp: String;
    Cmd: String;
    TGC: TGameClient;
    I:Integer;
    SC: TSSClient;
    Authed: Boolean;
    AuthList: String;
    AuthPassword: String;
begin
    // Check password
    AuthPassword := GS.GetString(VAR_ADMINPASSWORD, '');
    If (AuthPassword<>'') and (lowercase(Authpassword)<>lowercase(Password)) then begin
        // Log the event
        Writeln(GC.RealName+': sent rcon "'+Command+'" (wrong password)');
        Response := 'Wrong password';
        exit;
    end;

    // Check if player is authenticated
    AuthList := GS.GetString(VAR_ADMINLIST, '');
    UniqueID := GC.UniqueID;
    Authed := Pos(lowercase(UniqueId)+' ', lowercase(AuthList)+' ')>0;
    If not Authed then begin
        // Log the event
        Writeln(GC.RealName+': sent rcon "'+Command+'" (not authed)');
        Response := 'You are not allowed to do that';
        exit;
    end;

    // Log the event
    Writeln(GC.realname+': sent rcon "'+Command+'"');

    // Get the command
    I:=Pos(' ', Command);
    If (I<=0) then begin
        Cmd := Command;
        Command := '';
    end else begin
        Cmd := copy(command, 1, I-1);
        Command := copy(command, I+1, Length(Command)-I);
        Command := StrRep(command, ';' ,':');
        Command := StrRep(command, '`' ,'');
        Command := StrRep(command, '''' ,'');
    end;

    if (cmd='rcon') then
    begin
        Response := execute('gameserver."'+GS.name+'".rcon.'+command,nil);
        exit;
    end;

    if (cmd='say') then
    begin
        Response := execute('gameserver."'+GS.name+'".say '+command,nil);
        exit;
    end;

    if (cmd='restart') then
    begin
        If (GS.GameType = GT_HL2) or (GS.GameType = GT_CSS) then
            Response := execute('gameserver."'+GS.name+'".rcon.sv_restart 1',nil)
        else
            Response := execute('gameserver."'+GS.name+'".rcon.sv_restart 1',nil);
        exit;
    end;

    if (cmd='authed') then
    begin
        Response := execute('gameserver."'+GS.name+'".all.ssclient.var.list',nil);
        exit;
    end;

    if (cmd='shot') then
    begin
        Response := '';
        GC := GS.FindByUniqueID(command);
        If GC=nil then GC := GS.FindByUserID(StrToInt(command));
        If GC=nil then GC := GS.FindByName(command);
        if GC<>nil then TakeShot(GC, Response) else
        for i:=0 to GS.Count-1 do
            If WildCompare(GS.GameClient[I].Name, Command) Then
                TakeShot(GS.GameClient[I], Response);
        if Response='' then Response := 'Player not found';
        exit;
    end;

    if (cmd='map') or (cmd='changelevel') then
    begin
        Response := execute('gameserver."'+GS.name+'".rcon.changelevel '+command,nil);
        exit;
    end;

    if (Cmd='kick') and (command='enable') then
    begin
        Response := execute('gameserver."'+GS.name+'".var.set "OnPlayerUnauthed" "%player.kick `var.KickMessage''"', nil);
        exit;
    end;

    If (Cmd='kick') and (command='status') then
    begin
        //Tmp := GS.GetString('OnPlayerUnauthed', '""');
        Tmp := execute('gameserver."'+GS.name+'".var.OnPlayerUnauthed', nil);
        If (tmp='Unset variable') then Tmp := execute('gameserver.var.OnPlayerUnauthed', nil);
        If Tmp='"%player.kick `var.KickMessage''"' then Response := 'Kick enabled'
        else If (Tmp='";"') or (TMP='""') then Response := 'Kick disabled'
        else Response := 'Status unknown';
        exit;
    end;

    if (Cmd='kick') and (command='disable') then
    begin
        Response := execute('gameserver."'+GS.name+'".var.set "OnPlayerUnauthed" ";"', nil);
        exit;
    end;

    if (cmd='kick') then
    begin
        Response := '';
        TGC := GS.FindByUniqueID(uppercase(command));
        If TGC=nil then TGC := GS.FindByUserID(StrToInt(command));
        If TGC=nil then TGC := GS.FindByName(command);
        if TGC<>nil then KickPlayer(TGC, GC, Response) else
        for i:=0 to GS.Count-1 do
            If WildCompare(GS.GameClient[I].Name, Command) Then
                KickPlayer(GS.GameClient[I], GC, Response);
        if Response='' then Response := 'Player not found';
        exit;
    end;


    If (Cmd='help') or (Cmd='') then begin
        Response := Help;
        exit;
    end;
    Response := 'Hello '+GC.RealName+' your Steam_ID is '+UniqueID+#13#10+Help+#13#10'Unknown command '+cmd;
end;

// Called when client sends rcon command to localhost:27004 securely over ssclient tunneling
Procedure OnClientRconCommand(SC: TSSClient; Password, Command: String; var Response: String);
var AuthPassword: String;
    GS: TGameServer;
    GC: TGameClient;
begin
    GC := SC.GameClient;
    If GC=nil then begin
        // Log the event
        Writeln(SC.name+': sent rcon "'+Command+'" (not authed)');
        Response := 'You are not authed';
        exit;
    end;
    GS := TGameServer(SC.GetGameServer);
    If GS=nil then begin
        // Log the event
        Writeln(SC.name+': sent rcon "'+Command+'" (not authed)');
        Response := 'You are not authed';
        exit;
    end;

    // Check password
    AuthPassword := GS.GetString(VAR_ADMINPASSWORD, '');
    If (AuthPassword<>'') and (lowercase(Authpassword)<>lowercase(Password)) then begin
        // Log the event
        Writeln(SC.name+': sent rcon "'+Command+'" (wrong password)');
        Response := 'Wrong password';
        exit;
    end;
    HandleClient(GC, GS, Command, Password, Response);
end;

// Find the player with IP and Port
Function FindPlayer(IP:String; Port: Integer; Var OnServer: TGameServer): TGameClient;
var GSList: TGameServerList;
    GS: TGameServer;
    GC: TGameClient;
    SC: TssClient;
    I, J:Integer;
begin
    Result := nil;

    GSList := GetGameServerList;
    For I:=0 to GSList.count-1 do
    begin
        GS := GSList.GameServer[I];
        For J:=0 to GS.count-1 do
        begin
            GC := GS.GameClient[J];
            SC := TSSClient(GC.ssClient);
            If (GC.Variable.VarString[GC_IP]=IP) and (GC.Variable.VarInteger[GC_PORT]=Port) then
            begin
                OnServer := GS;
                Result := GC;
                exit;
            end;
            If (SC<>nil) then
            begin
                If (SC.Variable.VarString[SC_CONN_LOCALIP]=IP) and (SC.Variable.VarInteger[SC_CONN_LOCALPORT]=Port) then
                begin
                    OnServer := GS;
                    Result := GC;
                    exit;
                end;
            end;
        end;
    end;
end;

// Called when ssServer rcon server receives a rcon command
Procedure OnRConCommand(Sender: TRconServer; Password, Command, IP: String; Port: Integer; var Response: String);
var GC: TGameClient;
    GS: TGameServer;
begin
    command := trim(command);
    // Find the player
    GC := FindPlayer(IP, Port, GS);
    If (GC=nil) then begin
        // Log the event
        Writeln(IP+':'+inttostr(port)+': sent rcon "'+Command+'" (not authed)');
        Response := 'You are not authed';
        exit;
    end;

    HandleClient(GC, GS, Command, Password, Response);
end;

var RCon: THlRcon;
    reply: String;

begin
end.
