program soundcheck;
(*
 *  Soundcheck script for ssServer. Give filename to custom files as a parameter
 *  for this script. Example: Script.run SoundCheck scripts/soundcheck.pas soundcheck.txt
 *
 *  Use soundcheck_dod.txt for DOD
 *
 *  You must configure the action on how game servers respond when custom
 *  files are found in variable gameserver.<name>.var.set SoundCheckAction
 *  example: gameserver.<name>.var.set SoundCheckAction "LOG NAG KICK"
 *
 *  You must configure precache action var.PrecacheKick. Set it to either
 *  "true" or to "false". If Precache kick is true players are kicked if there
 *  is possibility that players have had sounds precached. The kick message is
 *  "You must start ssClient before half-life"
 *
 *  You can also set default behaviour in gameserver.var.SoundCheckAction
 *  example: gameserver.var.SoundCheckAction "LOG NAG"
 *
 *  You must also set a way to log the events of this script on file.
 *  Here is an example how to log the events on file and on console:
 * Example:

 Script.unload SoundCheck
 Script.run SoundCheck scripts/soundcheck.pas soundcheck.txt "\counter-strike\cstrike\sound\" "\counter-strike\cstrike\sprites\"
 Script.SoundCheck.log.add console stdout
 Script.SoundCheck.log.console.enable
 Script.SoundCheck.log.add FileLog file log/soundcheck.log
 Script.SoundCheck.log.FileLog.TimeStamp enable
 Script.SoundCheck.log.FileLog.TimeStampFormat "dd.MM.yy HH:mm:ss"
 Script.SoundCheck.log.FileLog.enable

 *)

var ScanTimer: TThreadTimer;
    CheatList: TStringPairTree;    // Short list of files found
    CheatMessage: TStringPairTree; // Longer description of the files
    ScanDirs: TStringArray;

(* How often custom files directory is scanned *)
Const ScanInterval = 16; // Seconds
Const PollInterval = 1;  // How often poll clients wheather hl is on
      VAR_SOUND_CHECK_ACTION = 'SoundCheckAction';

      // If this is set to true player is kicked if there is possibility
      // that half-life has precached the files.
      // Half-life loads the file into memory. After that player can
      // move customized sound files elsewhere and thus fool the scanner.

      // if this boolean is set to true player is required to start
      // the ssclient before starting the game
      PRECACHE_KICK = 'PrecacheKick';

(* Format information about authenticated client *)
Function GetClientInfo(Client: TssClient): String;
var Player: TGameClient;
    Server: TGameServer;
begin
    Result := '';
    If not Client.IsAuthed then exit;
    Player := Client.GameClient;
    Server := TGameServer(Client.GameClient.Parent);

    Result :=
        'Cheat: **ILLEGAL CUSTOM FILES FOUND**'+#10+
        'Player Name: '+Player.RealName+#10+
        'Player UniqueID: '+Player.Variable.VarString[GC_UNIQUEID]+#10+
        'Player Server: '+Server.Variable.VarString[GS_SERVERNAME]+#10+
        'Player SystemID: '+Client.Variable.VarString[SC_SYSTEMID]+#10+
        'Player IP: '+Player.Variable.VarString[GC_IP]+#10;
end;

Procedure HandleCheater(Client: TssClient; Cheats, CheatMessage: String);
var GC: TGameClient;
    GS: TGameServer;
    Action, Error: String;
    PlayerInfo: String;
begin
    If Client.IsAuthed then begin
        GC := Client.GameClient;
        GS := TGameServer(Client.GetGameServer);
        // Get Server's desired action
        Action := lowercase( GS.GetString(VAR_SOUND_CHECK_ACTION, 'log') );

        // Write to Game server
        if (pos('nag', action)>0) then
            //GS.DoSay(Client.name+' Custom file '+filename+' found', Error);
            GS.DoSay(GC.name+' illegal file found '+Cheats, Error);

        // Kick the player
        if (pos('kick', action)>0) then
            GC.KickMe('Custom files found '+Cheats);

        // Write to log
        if (pos('log', action)>0) then begin
            PlayerInfo := GetClientInfo(Client);
            Writeln(PlayerInfo+CheatMessage+#10+#10);
        end;
    end else
    begin
        // The Client is not authenticated yet,
        // Lets write something to the log
        // Write to log
        if (pos('log', action)>0) then
            Writeln(CheatMessage+#10+#10);
    end;
end;

Procedure   OnClientAuthed(Client: TssClient; Player: TGameClient; Server: TGameServer);
var Cheats, Msg: String;
    PrecacheKick: Boolean;
begin
    PrecacheKick := GetGlobalVariableList.VarBoolean[PRECACHE_KICK];
    if (PrecacheKick) and (client.Variable.VarBoolean[SC_PRECACHEPOSSIBLE]) then
    begin
        Player.KickMe('You must start ssClient before half-life');
        exit;
    end;

    // Client has been authenticated. Lets check if he has any cheats
    // buffered
    Cheats := CheatList.GetPair(Inttostr(Client.ID));
    Msg   := CheatMessage.GetPair(Inttostr(Client.ID));
    If Cheats='' then exit;
    // This client has cheated
    HandleCheater(Client, Cheats, Msg);
end;

Procedure OnClientDisconnected(Client: TssClient);
begin
    // Client disconnects. We can clear our table references
    CheatList.RemovePair(Inttostr(Client.ID));
    CheatMessage.RemovePair(Inttostr(Client.ID));
end;

(* Remember a cheat related to a unauthenticated client *)
Procedure RememberCheat(ID:String; Cheat, Msg: String);
begin
    CheatList.SetPair(ID, CheatList.GetPair(ID)+Cheat+' ');
    CheatMessage.SetPair(ID, CheatMessage.GetPair(ID)+Msg);
end;

Procedure   OnClientIllegalFile(Client: TssClient; FileName: String; FileSize, WavLength: Integer; FileChecksum: String; FileDate: String);
Var Msg: String;
begin
    // Information about the cheat
    Msg :=
        'Custom File Filename: '+filename+#10+
        'Custom File Size: '+inttostr(FileSize)+#10+
        'Custom File WavLength: '+inttostr(FileSize)+#10+
        'Custom File Checksum: '+FileChecksum+#10
        'Custom File Date: '+FileDate+#10;
    // If the client is authenticated write the information right away
    If Client.IsAuthed then begin
        // Write this cheat up right now
        HandleCheater(Client, filename, Msg);
    end else
    // If the client is not unauthenticated, remember his cheats
    begin
        // Remeber this cheat until the client is authenticated
        RememberCheat(Inttostr(Client.ID), filename, Msg);
    end;
end;

Procedure OnTimer(Sender: TThreadTimer);
var I, J:Integer;
    ss: TssServer;
    SC: TssClient;
begin
    If Sender=ScanTimer then begin
        ss := GetssServer;
        For I:=0 to ss.count-1 do begin
            SC := ss.Client[I];
            For J:=0 to GetArrayLength(ScanDirs)-1 do
                SC.QueryCustomSounds(ScanDirs[J]);
        end;
        exit;
    end;
end;

Procedure   Initialization;
var CustomFileList, ErrorMsg, FileName: String;
    I:Integer;
begin
    CheatList := TStringPairTree.Create(False);
    CheatMessage := TStringPairTree.Create(False);
    If ParamCount<1 then begin
        Writeln('Soundcheck file required as parameter');
        Unload;
        Exit;
    end;

    If ParamCount<2 then begin
        Writeln('Scan directory required as second parameter');
        Unload;
        Exit;
    end;

    Writeln('***Executing soundcheck script ***');

    Writeln('1) Setting scan directories');
    FileName := ParamStr(1);
    SetArrayLength(ScanDirs, ParamCount-1);
    For I:=0 to ParamCount-2 do begin
        ScanDirs[I] := ParamStr(I+2);
        Writeln('   * Scan directory: '+ScanDirs[I]);
    end;


    // Load list on illegal files from a file
    Writeln('2) Loading list of illegal custom files: '+FileName);
    CustomFileList := LoadFile(FileName, ErrorMsg);
    If (CustomFileList='') then begin
        Writeln('No files loaded');
        Unload;
        Exit;
    end;
    if (ErrorMsg<>'') then
    begin
        Writeln('Failed to load file '+Filename+' '+ErrorMsg);
        Unload;
        exit;
    end;

    Writeln('3) Scanning on timed intervals');
    ScanTimer := CreateTimer;
    ScanTimer.interval := 1000*ScanInterval;
    ScanTimer.enabled := True;

    GetssServer.LoadIllegalFiles(CustomFileList);
end;

Procedure   Finalization;
begin
    DestroyTimer(ScanTimer);
    CheatList.Destroy;
    CheatMessage.Destroy;
    ScanTimer := nil;
    GetssServer.LoadIllegalFiles('');
    Writeln('Soundcheck script unloaded');
end;

begin
end.
